import click
from exasol_integration_test_docker_environment.lib.utils.cli_function_decorators import (
    add_options,
)

import exasol.slc_ci.lib.prepare_test_container as lib_prepare_test_container
from exasol.slc_ci.cli.cli import cli
from exasol.slc_ci.cli.options.branch_options import commit_sha_option
from exasol.slc_ci.cli.options.docker_options import docker_options
from exasol.slc_ci.lib.ci_prepare import CIPrepare
from exasol.slc_ci.lib.ci_push_test_container import CIPushTestContainer


@cli.command()
@add_options([commit_sha_option])
@add_options(docker_options)
def prepare_test_container(
    commit_sha: str,
    docker_user: str,
    docker_password: str,
) -> None:
    ci_push_test_container = CIPushTestContainer()
    ci_prepare = CIPrepare()
    lib_prepare_test_container.prepare_test_container(
        commit_sha=commit_sha,
        docker_user=docker_user,
        docker_password=docker_password,
        ci_push_test_container=ci_push_test_container,
        ci_prepare=ci_prepare,
    )



import logging
from typing import Callable, Set

from exasol_script_languages_container_ci.lib.branch_config import BranchConfig
from exasol_script_languages_container_ci.lib.ci_build import CIBuild
from exasol_script_languages_container_ci.lib.ci_prepare import CIPrepare
from exasol_script_languages_container_ci.lib.ci_push import CIPush
from exasol_script_languages_container_ci.lib.ci_security_scan import CISecurityScan
from exasol_script_languages_container_ci.lib.ci_test import CIExecuteTest
from exasol_script_languages_container_ci.lib.config.config_data_model import Config
from exasol_script_languages_container_ci.lib.git_access import GitAccess


def get_all_affected_files(git_access: GitAccess, base_branch: str) -> Set[str]:
    base_last_commit_sha = git_access.get_head_commit_sha_of_branch(base_branch)
    changed_files = set()  # type: ignore
    for commit in git_access.get_last_commits():
        if commit == base_last_commit_sha:
            break
        changed_files.update(git_access.get_files_of_commit(commit))
    return changed_files


def check_if_need_to_build(
    branch_name: str, config: Config, flavor: str, git_access: GitAccess
):
    if BranchConfig.build_always(branch_name):
        return True
    if "[rebuild]" in git_access.get_last_commit_message():
        return True
    affected_files = list(get_all_affected_files(git_access, config.build.base_branch))
    logging.debug(
        f"check_if_need_to_build: Found files of last commits: {affected_files}"
    )
    for ignore_path in config.build.ignore.paths:
        affected_files = list(
            filter(lambda file: not file.startswith(ignore_path), affected_files)
        )

    if len(affected_files) > 0:
        # Now filter out also other flavor folders
        this_flavor_path = f"flavors/{flavor}"
        affected_files = list(
            filter(
                lambda file: not file.startswith("flavors")
                or file.startswith(this_flavor_path),
                affected_files,
            )
        )
    logging.debug(f"check_if_need_to_build: filtered files: {affected_files}")
    return len(affected_files) > 0


def ci(
    flavor: str,
    branch_name: str,
    docker_user: str,
    docker_password: str,
    docker_build_repository: str,
    docker_release_repository: str,
    commit_sha: str,
    build_config: Config,
    git_access: GitAccess,
    ci_build: CIBuild = CIBuild(),
    ci_execute_tests: CIExecuteTest = CIExecuteTest(),
    ci_push: CIPush = CIPush(),
    ci_security_scan: CISecurityScan = CISecurityScan(),
    ci_prepare: CIPrepare = CIPrepare(),
):
    """
    Run CI build:
    1. Build image
    2. Run db tests
    3. Run security scan
    4. Push to docker repositories
    """
    logging.info(f"Running CI build for parameters: {locals()}")

    flavor_path = (f"flavors/{flavor}",)
    test_container_folder = "test_container"
    rebuild = BranchConfig.rebuild(branch_name)
    needs_to_build = check_if_need_to_build(
        branch_name, build_config, flavor, git_access
    )
    if needs_to_build:
        ci_prepare.prepare()
        ci_build.build(
            flavor_path=flavor_path,
            rebuild=rebuild,
            build_docker_repository=docker_build_repository,
            commit_sha=commit_sha,
            docker_user=docker_user,
            docker_password=docker_password,
            test_container_folder=test_container_folder,
        )
        ci_execute_tests.execute_tests(
            flavor_path=flavor_path,
            docker_user=docker_user,
            docker_password=docker_password,
            test_container_folder=test_container_folder,
        )
        ci_security_scan.run_security_scan(flavor_path=flavor_path)
        ci_push.push(
            flavor_path=flavor_path,
            target_docker_repository=docker_build_repository,
            target_docker_tag_prefix=commit_sha,
            docker_user=docker_user,
            docker_password=docker_password,
        )
        ci_push.push(
            flavor_path=flavor_path,
            target_docker_repository=docker_build_repository,
            target_docker_tag_prefix="",
            docker_user=docker_user,
            docker_password=docker_password,
        )
        if BranchConfig.push_to_docker_release_repo(branch_name):
            ci_push.push(
                flavor_path=flavor_path,
                target_docker_repository=docker_release_repository,
                target_docker_tag_prefix="",
                docker_user=docker_user,
                docker_password=docker_password,
            )
    else:
        logging.warning(f"Skipping build...")



from .check_if_build_needed import check_if_build_needed
from .export_and_scan_vulnerabilities import export_and_scan_vulnerabilities
from .get_build_runner import get_build_runner
from .get_flavors import get_flavors
from .get_test_matrix import get_test_matrix
from .prepare_test_container import prepare_test_container
from .run_tests import run_tests



import os
from datetime import datetime
from pathlib import Path

from exasol_integration_test_docker_environment.cli.options.system_options import (
    DEFAULT_OUTPUT_DIRECTORY,
)
from exasol_integration_test_docker_environment.lib.logging import luigi_log_config


class CIPrepare:

    def prepare(self, commit_sha: str):
        log_path = Path(DEFAULT_OUTPUT_DIRECTORY) / "jobs" / "logs" / "main.log"
        log_path.parent.mkdir(parents=True, exist_ok=True)
        os.environ[luigi_log_config.LOG_ENV_VARIABLE_NAME] = f"{log_path.absolute()}"

        security_scan_path = Path(DEFAULT_OUTPUT_DIRECTORY) / "security_scan"
        meta_data_path = Path(DEFAULT_OUTPUT_DIRECTORY) / "meta_data"
        security_scan_path.mkdir(parents=True, exist_ok=True)
        meta_data_path.mkdir(parents=True, exist_ok=True)

        with open(meta_data_path / "start_date", "w") as f:
            f.write(datetime.now().isoformat())

        with open(meta_data_path / "commit_sha", "w") as f:
            f.write(commit_sha)



import os
from pathlib import Path

from exasol_integration_test_docker_environment.cli.options.system_options import (
    DEFAULT_OUTPUT_DIRECTORY,
)
from exasol_integration_test_docker_environment.lib.logging import luigi_log_config


class CIPrepare:

    def prepare(self):
        log_path = Path(DEFAULT_OUTPUT_DIRECTORY) / "jobs" / "logs" / "main.log"
        log_path.parent.mkdir(parents=True, exist_ok=True)
        os.environ[luigi_log_config.LOG_ENV_VARIABLE_NAME] = f"{log_path.absolute()}"



from exasol.slc_ci.lib.ci_prepare import CIPrepare
from exasol.slc_ci.lib.ci_push_test_container import CIPushTestContainer
from exasol.slc_ci.lib.get_build_config_model import get_build_config_model


def prepare_test_container(
    docker_user: str,
    docker_password: str,
    commit_sha: str,
    ci_prepare: CIPrepare,
    ci_push_test_container: CIPushTestContainer,
) -> None:
    build_config = get_build_config_model()
    ci_prepare.prepare(commit_sha=commit_sha)

    ci_push_test_container.push_test_container(
        build_docker_repository=build_config.docker_build_repository,
        force=True,
        commit_sha=commit_sha,
        docker_user=docker_user,
        docker_password=docker_password,
        test_container_folder=build_config.test_container_folder,
    )



import click

branch_option = click.option(
    "--branch-name",
    type=str,
    required=True,
    help="In case of a PR, the source branch of the PR.",
)

base_branch_option = click.option(
    "--base-ref",
    type=str,
    required=True,
    help="In case of a PR, the target ref of the PR.",
)

commit_sha_option = click.option(
    "--commit-sha",
    type=str,
    required=True,
    help="Commit sha that trigger the GH event",
)

remote_option = click.option(
    "--remote",
    type=str,
    default="origin",
    required=False,
    help="The remote Git repository of the base branch",
)



import logging
from pathlib import Path
from typing import Tuple

from exasol.slc.api import export

from exasol.slc_ci.lib.ci_step_output_printer import (
    CIStepOutputPrinter,
    CIStepOutputPrinterProtocol,
)


class CIExport:

    def __init__(
        self, printer: CIStepOutputPrinterProtocol = CIStepOutputPrinter(logging.info)
    ):
        self._printer = printer

    def export(
        self, flavor_path: Tuple[str, ...], goal: str, output_directory: str
    ) -> Path:
        """
        Export the flavor as tar.gz file.
        The returned path is the path of the tar.gz file.
        """

        logging.info(f"Running command 'export' with parameters: {locals()}")
        export_result = export(
            flavor_path=flavor_path,
            workers=7,
            log_level="WARNING",
            use_job_specific_log_file=True,
            release_goal=(goal,),
            output_directory=output_directory,
        )
        self._printer.print_exasol_docker_images()
        export_infos = list(export_result.export_infos.values())
        if len(export_infos) != 1:
            raise RuntimeError(f"Unexpected number of export infos")
        export_info = export_infos[0]
        export_flavor_infos = list(export_info.values())
        if len(export_flavor_infos) != 1:
            raise RuntimeError(f"Unexpected number of export flavor infos")

        export_flavor_info = export_flavor_infos[0]
        return Path(export_flavor_info.cache_file)



from pathlib import Path

from exasol.slc_ci.model.build_config_model import BuildConfig


def _find_build_config_path() -> Path:
    """
    Traverse paths upwards until starting from cwd until finding "build_config.json".
    If not found, raises FileNotFoundError.
    """
    base_path = Path.cwd()
    for test_paths in [base_path] + list(base_path.parents):
        current_build_config_path = test_paths / "build_config.json"
        if current_build_config_path.exists():
            return current_build_config_path
    raise FileNotFoundError("Could not find build config file")


def get_build_config_model() -> BuildConfig:
    build_config_path = _find_build_config_path()
    return BuildConfig.model_validate_json(build_config_path.read_text(), strict=True)



import logging
from pathlib import Path
from typing import Optional, Protocol, Tuple

from exasol.slc.api.run_db_tests import run_db_test
from exasol.slc.models.accelerator import Accelerator
from exasol.slc.models.test_result import AllTestsResult

from exasol.slc_ci.lib.ci_step_output_printer import (
    CIStepOutputPrinter,
    CIStepOutputPrinterProtocol,
)


class DBTestRunnerProtocol(Protocol):
    def run(
        self,
        flavor_path: Tuple[str, ...],
        release_goal: Tuple[str, ...],
        test_folder: Tuple[str, ...],
        test_container_folder: str,
        generic_language_tests: Tuple[str, ...],
        accelerator: Accelerator,
        source_docker_tag_prefix: str,
        source_docker_repository_name: str,
        workers: int,
        docker_username: Optional[str],
        docker_password: Optional[str],
        use_existing_container: Optional[str],
    ) -> AllTestsResult:
        raise NotImplementedError()


class DBTestRunner(DBTestRunnerProtocol):
    def run(
        self,
        flavor_path: Tuple[str, ...],
        release_goal: Tuple[str, ...],
        test_folder: Tuple[str, ...],
        test_container_folder: str,
        generic_language_tests: Tuple[str, ...],
        accelerator: Accelerator,
        source_docker_tag_prefix: str,
        source_docker_repository_name: str,
        workers: int,
        docker_username: Optional[str],
        docker_password: Optional[str],
        use_existing_container: Optional[str],
    ) -> AllTestsResult:
        return run_db_test(
            flavor_path=flavor_path,
            release_goal=release_goal,
            test_folder=test_folder,
            test_container_folder=test_container_folder,
            generic_language_test=generic_language_tests,
            accelerator=accelerator,
            source_docker_tag_prefix=source_docker_tag_prefix,
            source_docker_repository_name=source_docker_repository_name,
            workers=workers,
            source_docker_username=docker_username,
            source_docker_password=docker_password,
            log_level="WARNING",
            use_job_specific_log_file=True,
            use_existing_container=use_existing_container,
        )


class CIExecuteTest:

    def __init__(
        self,
        db_test_runner: DBTestRunnerProtocol = DBTestRunner(),
        printer: CIStepOutputPrinterProtocol = CIStepOutputPrinter(logging.info),
    ):
        self._db_test_runner = db_test_runner
        self._printer = printer

    def execute_tests(
        self,
        flavor_path: Tuple[str, ...],
        slc_path: Path,
        goal: str,
        test_folder: str,
        generic_language_tests: Tuple[str, ...],
        accelerator: Accelerator,
        docker_user: str,
        docker_password: str,
        test_container_folder: str,
        commit_sha: str,
        build_docker_repository: str,
    ):
        """
        Run db tests
        """
        db_tests_are_ok = self._run_db_tests(
            flavor_path=flavor_path,
            goal=goal,
            slc_path=slc_path,
            test_folder=test_folder,
            generic_language_tests=generic_language_tests,
            accelerator=accelerator,
            docker_user=docker_user,
            docker_password=docker_password,
            test_container_folder=test_container_folder,
            commit_sha=commit_sha,
            build_docker_repository=build_docker_repository,
        )
        self._printer.print_exasol_docker_images()
        if not db_tests_are_ok:
            raise AssertionError("Not all tests are ok!")

    def _run_db_tests(
        self,
        flavor_path: Tuple[str, ...],
        slc_path: Path,
        goal: str,
        test_folder: str,
        generic_language_tests: Tuple[str, ...],
        accelerator: Accelerator,
        docker_user: str,
        docker_password: str,
        test_container_folder: str,
        commit_sha: str,
        build_docker_repository: str,
    ) -> bool:
        logging.info(f"Running command 'run_db_test' for flavor-path {flavor_path}")
        db_test_result = self._db_test_runner.run(
            flavor_path=flavor_path,
            test_folder=(test_folder,),
            release_goal=(goal,),
            generic_language_tests=generic_language_tests,
            accelerator=accelerator,
            source_docker_tag_prefix=commit_sha,
            source_docker_repository_name=build_docker_repository,
            workers=7,
            docker_username=docker_user,
            docker_password=docker_password,
            test_container_folder=test_container_folder,
            use_existing_container=str(slc_path),
        )
        self._printer.print_file(db_test_result.command_line_output_path)
        return db_test_result.tests_are_ok



import click
from exasol_integration_test_docker_environment.lib.utils.cli_function_decorators import (
    add_options,
)

import exasol.slc_ci.lib.run_tests as lib_run_tests
from exasol.slc_ci.cli.cli import cli
from exasol.slc_ci.cli.options.branch_options import commit_sha_option
from exasol.slc_ci.cli.options.docker_options import docker_options
from exasol.slc_ci.cli.options.flavor_options import flavor_options
from exasol.slc_ci.cli.options.test_options import test_set_options
from exasol.slc_ci.lib.ci_prepare import CIPrepare
from exasol.slc_ci.lib.ci_test import CIExecuteTest


@cli.command()
@add_options(flavor_options)
@add_options(
    [
        click.option(
            "--slc-directory",
            type=str,
            required=True,
            help="Directory where existing SLC file is stored.",
        ),
    ]
)
@add_options(test_set_options)
@add_options(docker_options)
@add_options([commit_sha_option])
def run_tests(
    flavor: str,
    slc_directory: str,
    test_set_name: str,
    docker_user: str,
    docker_password: str,
    commit_sha: str,
) -> None:
    ci_prepare = CIPrepare()
    ci_test = CIExecuteTest()
    lib_run_tests.run_tests(
        flavor=flavor,
        slc_directory=slc_directory,
        test_set_name=test_set_name,
        docker_user=docker_user,
        docker_password=docker_password,
        ci_prepare=ci_prepare,
        ci_test=ci_test,
        commit_sha=commit_sha,
    )



import logging
from pathlib import Path

from exasol.slc_ci.lib.ci_prepare import CIPrepare
from exasol.slc_ci.lib.ci_test import CIExecuteTest
from exasol.slc_ci.lib.get_build_config_model import get_build_config_model
from exasol.slc_ci.lib.get_flavor_ci_model import get_flavor_ci_model


def run_tests(
    flavor: str,
    slc_directory: str,
    test_set_name: str,
    docker_user: str,
    docker_password: str,
    commit_sha: str,
    ci_prepare: CIPrepare = CIPrepare(),
    ci_test: CIExecuteTest = CIExecuteTest(),
) -> None:
    logging.info(f"Run tests for parameters: {locals()}")
    build_config = get_build_config_model()
    flavor_config = get_flavor_ci_model(build_config, flavor)
    matched_test_set = [
        test_set
        for test_set in flavor_config.test_config.test_sets
        if test_set.name == test_set_name
    ]
    if len(matched_test_set) != 1:
        raise ValueError(f"Invalid test set name: {test_set_name}")
    test_set_folders = [folder for folder in matched_test_set[0].folders]
    goal = matched_test_set[0].goal
    generic_language_tests = matched_test_set[0].generic_language_tests
    accelerator = matched_test_set[0].accelerator
    slc_path = Path(slc_directory)
    if not slc_path.exists():
        raise ValueError(f"{slc_path} does not exist")
    slc_files = list(slc_path.glob(f"{flavor}*.tar.gz"))
    if len(slc_files) != 1:
        raise ValueError(
            f"{slc_directory} does not contain expected tar.gz file, but \n {slc_files}"
        )
    slc_file_path = slc_files[0]

    flavor_path = (f"{build_config.flavors_path}/{flavor}",)
    test_container_folder = build_config.test_container_folder

    ci_prepare.prepare(commit_sha=commit_sha)
    for test_folder in test_set_folders:
        ci_test.execute_tests(
            flavor_path=flavor_path,
            slc_path=slc_file_path,
            goal=goal,
            test_folder=test_folder,
            generic_language_tests=tuple(),
            accelerator=accelerator,
            test_container_folder=test_container_folder,
            docker_user=docker_user,
            docker_password=docker_password,
            commit_sha=commit_sha,
            build_docker_repository=build_config.docker_build_repository,
        )

    if generic_language_tests:
        ci_test.execute_tests(
            flavor_path=flavor_path,
            slc_path=slc_file_path,
            goal=goal,
            test_folder="",
            generic_language_tests=tuple(generic_language_tests),
            accelerator=accelerator,
            test_container_folder=test_container_folder,
            docker_user=docker_user,
            docker_password=docker_password,
            commit_sha=commit_sha,
            build_docker_repository=build_config.docker_build_repository,
        )



from exasol_integration_test_docker_environment.lib.utils.cli_function_decorators import (
    add_options,
)

import exasol.slc_ci.lib.get_build_runner as lib_get_build_runner
from exasol.slc_ci.cli.cli import cli
from exasol.slc_ci.cli.options.flavor_options import flavor_options
from exasol.slc_ci.cli.options.github_options import github_options
from exasol.slc_ci.lib.github_access import GithubAccess


@cli.command()
@add_options(flavor_options)
@add_options(github_options)
def get_build_runner(flavor: str, github_output_var: str):
    github_access = GithubAccess(github_output_var)
    lib_get_build_runner.get_build_runner(flavor=flavor, github_access=github_access)



import logging
from typing import Tuple

from exasol.slc.api.push import push

from exasol_script_languages_container_ci.lib.ci_step_output_printer import (
    CIStepOutputPrinter,
    CIStepOutputPrinterProtocol,
)


class CIPush:

    def __init__(
        self, printer: CIStepOutputPrinterProtocol = CIStepOutputPrinter(logging.info)
    ):
        self._printer = printer

    def push(
        self,
        flavor_path: Tuple[str, ...],
        target_docker_repository: str,
        target_docker_tag_prefix: str,
        docker_user: str,
        docker_password: str,
    ):
        """
        Push the docker image to Dockerhub
        """

        logging.info(f"Running command 'push' with parameters: {locals()}")
        push(
            flavor_path=flavor_path,
            push_all=True,
            force_push=True,
            workers=7,
            target_docker_repository_name=target_docker_repository,
            target_docker_tag_prefix=target_docker_tag_prefix,
            target_docker_username=docker_user,
            target_docker_password=docker_password,
            log_level="WARNING",
            use_job_specific_log_file=True,
        )
        self._printer.print_exasol_docker_images()



import json
from contextlib import contextmanager
from inspect import cleandoc
from pathlib import Path
from typing import Callable

import docker


@contextmanager
def get_config(config_file: str):
    """
    Opens config file and returns parsed JSON object.
    """
    with open(config_file) as f:
        yield json.load(f)



import logging
from typing import Optional, Tuple

from exasol.slc.api import build

from exasol.slc_ci.lib.ci_step_output_printer import (
    CIStepOutputPrinter,
    CIStepOutputPrinterProtocol,
)


class CIBuild:

    def __init__(
        self, printer: CIStepOutputPrinterProtocol = CIStepOutputPrinter(logging.info)
    ):
        self._printer = printer

    def build(
        self,
        flavor_path: Tuple[str, ...],
        rebuild: bool,
        build_docker_repository: Optional[str],
        docker_user: str,
        docker_password: str,
    ):
        """
        Build the script-language container for given flavor.
        """

        logging.info(f"Running command 'build' with parameters: {locals()}")
        if build_docker_repository is None:
            slc_image_infos = build(
                flavor_path=flavor_path,
                force_rebuild=rebuild,
                source_docker_username=docker_user,
                source_docker_password=docker_password,
                shortcut_build=False,
                workers=7,
                log_level="WARNING",
                use_job_specific_log_file=True,
            )
        else:
            slc_image_infos = build(
                flavor_path=flavor_path,
                force_rebuild=rebuild,
                source_docker_repository_name=build_docker_repository,
                source_docker_username=docker_user,
                source_docker_password=docker_password,
                shortcut_build=False,
                workers=7,
                log_level="WARNING",
                use_job_specific_log_file=True,
            )
        self._printer.print_exasol_docker_images()



import click
from exasol_integration_test_docker_environment.lib.utils.cli_function_decorators import (
    add_options,
)

import exasol.slc_ci.lib.export_and_scan_vulnerabilities as lib_export_and_scan_vulnerabilities
from exasol.slc_ci.cli.cli import cli
from exasol.slc_ci.cli.options.branch_options import branch_option, commit_sha_option
from exasol.slc_ci.cli.options.docker_options import docker_options
from exasol.slc_ci.cli.options.flavor_options import flavor_options
from exasol.slc_ci.cli.options.github_options import github_options
from exasol.slc_ci.lib.ci_build import CIBuild
from exasol.slc_ci.lib.ci_export import CIExport
from exasol.slc_ci.lib.ci_prepare import CIPrepare
from exasol.slc_ci.lib.ci_push import CIPush
from exasol.slc_ci.lib.ci_security_scan import CISecurityScan
from exasol.slc_ci.lib.git_access import GitAccess
from exasol.slc_ci.lib.github_access import GithubAccess


@cli.command()
@add_options(flavor_options)
@add_options([branch_option, commit_sha_option])
@add_options(docker_options)
@add_options(github_options)
@add_options(
    [
        click.option(
            "--release",
            is_flag=True,
            help="Activate release mode and push SLC to the release Docker repository.",
        )
    ]
)
def export_and_scan_vulnerabilities(
    flavor: str,
    branch_name: str,
    docker_user: str,
    docker_password: str,
    commit_sha: str,
    github_output_var: str,
    release: bool,
) -> None:
    git_access: GitAccess = GitAccess()
    github_access: GithubAccess = GithubAccess(github_output_var)
    ci_build = CIBuild()
    ci_security_scan = CISecurityScan()
    ci_prepare = CIPrepare()
    ci_export = CIExport()
    ci_push = CIPush()

    lib_export_and_scan_vulnerabilities.export_and_scan_vulnerabilities(
        release=release,
        flavor=flavor,
        branch_name=branch_name,
        docker_user=docker_user,
        docker_password=docker_password,
        commit_sha=commit_sha,
        git_access=git_access,
        github_access=github_access,
        ci_build=ci_build,
        ci_security_scan=ci_security_scan,
        ci_prepare=ci_prepare,
        ci_export=ci_export,
        ci_push=ci_push,
    )



import logging
from typing import Tuple

from exasol.slc.api import export

from exasol_script_languages_container_ci.lib.ci_step_output_printer import (
    CIStepOutputPrinter,
    CIStepOutputPrinterProtocol,
)


class CIExport:

    def __init__(
        self, printer: CIStepOutputPrinterProtocol = CIStepOutputPrinter(logging.info)
    ):
        self._printer = printer

    def export(self, flavor_path: Tuple[str, ...], export_path: str):
        """
        Export the flavor as tar.gz file
        """

        logging.info(f"Running command 'push' with parameters: {locals()}")
        export_result = export(
            flavor_path=flavor_path,
            export_path=export_path,
            workers=7,
            log_level="WARNING",
            use_job_specific_log_file=True,
        )
        self._printer.print_exasol_docker_images()



import logging
from pathlib import Path

from github import Github, GithubException


class GithubReleaseAssetUploader:
    """
    Implements upload to a Github Release.
    See https://docs.github.com/en/rest/releases/assets#upload-a-release-asset for details.
    The access token needs to be stored in the environment variable GITHUB_TOKEN
    """

    def __init__(self, token):
        self._token = token

    def upload(
        self,
        archive_path: str,
        label: str,
        repo_id: str,
        release_id: int,
        content_type: str,
    ):
        gh = Github(self._token)
        gh_repo = gh.get_repo(repo_id)
        release = gh_repo.get_release(release_id)
        # Check GH limitation
        # https://docs.github.com/en/repositories/releasing-projects-on-github/about-releases#storage-and-bandwidth-quotas
        if Path(archive_path).stat().st_size >= 2 * (2**30):
            logging.error("File larger than 2GB. Skipping it...")
        else:
            try:
                release.upload_asset(
                    path=archive_path, label=label, content_type=content_type
                )
            except GithubException as ex:
                logging.error(
                    f"Upload of asset {archive_path} to release {release_id} failed: {ex}"
                )
                raise ex



from exasol_integration_test_docker_environment.lib.utils.cli_function_decorators import (
    add_options,
)

import exasol.slc_ci.lib.get_flavors as lib_get_flavors
from exasol.slc_ci.cli.cli import cli
from exasol.slc_ci.cli.options.github_options import github_options
from exasol.slc_ci.lib.github_access import GithubAccess


@cli.command()
@add_options(github_options)
def get_flavors(
    github_output_var: str,
):
    """
    Searches for all available flavors and writes result as JSON array to Github variable <github-output-var>.
    """
    github_access: GithubAccess = GithubAccess(github_output_var=github_output_var)
    lib_get_flavors.get_flavors(github_access=github_access)



from typing import Iterable

from git import Repo


class GitAccess:

    def get_last_commit_message(self):
        """
        Assumes that PWD belongs to a GIT repository. Get's the last commit message of this repo and returns it as string
        :return: Last commit message of current working directory GIT repository.
        """
        return Repo().head.commit.message

    def get_head_commit_sha_of_branch(self, branch_name) -> str:
        """
        Returns the last commit sha of given branch.
        :raise: ValueError: if the refs with label 'branch_name' does not exists or is not unique.
        """
        repo = Repo()
        branch = [b for b in repo.refs if b.name == branch_name]  # type: ignore
        if len(branch) == 0:
            ex_msg = f"Branch '{branch_name}' does not exist."
            raise ValueError(ex_msg)
        elif len(branch) > 1:
            ex_msg = f"Branch '{branch_name}' found more than once."
            raise ValueError(ex_msg)
        return str(branch[0].commit)

    def get_last_commits(self) -> Iterable[str]:
        """
        Returns all commit-sha's of the current branch of the repo in the cwd.
        """
        repo = Repo()
        for c in repo.iter_commits(repo.head):
            yield str(c)

    def get_files_of_commit(self, commit_sha) -> Iterable[str]:
        """
        Returns the files of the specific commits of the repo in the cwd.
        """
        repo = Repo()
        return repo.commit(commit_sha).stats.files.keys()  # type: ignore



import logging
from typing import Optional

from exasol.slc.internal.tasks.test.test_container_content import (
    build_test_container_content,
)
from exasol_integration_test_docker_environment.lib.api import push_test_container

from exasol.slc_ci.lib.ci_step_output_printer import (
    CIStepOutputPrinter,
    CIStepOutputPrinterProtocol,
)


class CIPushTestContainer:

    def __init__(
        self, printer: CIStepOutputPrinterProtocol = CIStepOutputPrinter(logging.info)
    ):
        self._printer = printer

    def push_test_container(
        self,
        build_docker_repository: Optional[str],
        force: bool,
        commit_sha: str,
        docker_user: str,
        docker_password: str,
        test_container_folder: str,
    ):
        """
        Push the test container with given commit SHA.
        """

        logging.info(
            f"Running command 'push_test_container' with parameters: {locals()}"
        )
        content = build_test_container_content(test_container_folder)
        test_container_image_infos = push_test_container(
            force_push=force,
            workers=7,
            test_container_content=content,
            target_docker_repository_name=build_docker_repository,
            target_docker_tag_prefix=commit_sha,
            target_docker_username=docker_user,
            target_docker_password=docker_password,
            log_level="WARNING",
            use_job_specific_log_file=True,
        )
        self._printer.print_exasol_docker_images()



import click

github_options = [
    click.option(
        "--github-output-var",
        type=str,
        required=True,
        help="Sets the github variable where the result of the operation will be stored.",
    )
]



import click


@click.group()
def cli():
    """
    EXASLC_CI - Exasol Script Languages Continuous Integration

    Provides a CLI to build/test/deploy and release Exasol's Script-Languages-Containters in a Github CI environment.

    Examples:

        Print this help message:

            $ exaslc-ci --help

        Get the list of available flavors and write to $GITHUB_OUT:

            $ exaslc-ci get-flavors --github-output-var flavors
    """
    pass



import logging
from pathlib import Path
from typing import Tuple

from exasol.slc.api import security_scan

from exasol_script_languages_container_ci.lib.ci_step_output_printer import (
    CIStepOutputPrinter,
    CIStepOutputPrinterProtocol,
)


class CISecurityScan:

    def __init__(
        self, printer: CIStepOutputPrinterProtocol = CIStepOutputPrinter(logging.info)
    ):
        self._printer = printer

    def run_security_scan(self, flavor_path: Tuple[str, ...]):
        """
        Run security scan and print result
        """

        logging.info(f"Running command 'security_scan' with parameters {locals()}")
        security_scan_result = security_scan(
            flavor_path=flavor_path,
            workers=7,
            log_level="WARNING",
            use_job_specific_log_file=True,
        )
        logging.info("============= SECURITY REPORT ===========")
        self._printer.print_file(Path(security_scan_result.report_path))
        self._printer.print_exasol_docker_images()
        if not security_scan_result.scans_are_ok:
            raise AssertionError("Some security scans not successful.")



# generated by datamodel-codegen:
#   filename:  <stdin>

from __future__ import annotations

from typing import List

from pydantic import BaseModel, Extra


class Ignore(BaseModel):
    class Config:
        extra = Extra.forbid

    paths: List[str]


class Build(BaseModel):
    class Config:
        extra = Extra.forbid

    ignore: Ignore
    base_branch: str


class Release(BaseModel):
    class Config:
        extra = Extra.forbid

    timeout_in_minutes: int


class Config(BaseModel):
    class Config:
        extra = Extra.forbid

    build: Build
    release: Release



import json
from typing import List

from exasol.slc_ci.lib.get_build_config_model import get_build_config_model
from exasol.slc_ci.lib.github_access import GithubAccess


def get_flavors(github_access: GithubAccess) -> None:
    build_config = get_build_config_model()
    if not build_config.flavors_path.exists():
        raise ValueError(f"Flavor path '{build_config.flavors_path}' does not exist")
    flavors: List[str] = list()
    for p in build_config.flavors_path.iterdir():
        if p.is_dir():
            flavors.append(p.name)

    github_access.write_result(json.dumps(flavors))



import logging

import click

from exasol_script_languages_container_ci.cli.cli import cli
from exasol_script_languages_container_ci.lib.ci import ci
from exasol_script_languages_container_ci.lib.config.config_data_model import Config
from exasol_script_languages_container_ci.lib.git_access import GitAccess


@cli.command()
@click.option("--flavor", required=True, type=str, help="Flavor name.")
@click.option("--branch-name", required=True, type=str, help="Branch name.")
@click.option("--docker-user", required=True, type=str, help="Docker user name")
@click.option("--docker-password", required=True, type=str, help="Docker password")
@click.option(
    "--docker-build-repository", required=True, type=str, help="Docker build repository"
)
@click.option(
    "--docker-release-repository",
    required=True,
    type=str,
    help="Docker release repository",
)
@click.option("--commit-sha", required=True, type=str, help="Commit SHA")
@click.option(
    "--config-file",
    required=True,
    type=click.Path(exists=True, file_okay=True, dir_okay=False),
    help="The build config file (project specific)",
)
@click.pass_context
def run_ci(
    ctx: click.Context,
    flavor: str,
    branch_name: str,
    docker_user: str,
    docker_password: str,
    docker_build_repository: str,
    docker_release_repository: str,
    commit_sha: str,
    config_file: str,
):
    logging.basicConfig(level=logging.INFO)
    build_config = Config.parse_file(config_file)
    ci(
        flavor=flavor,
        branch_name=branch_name,
        docker_user=docker_user,
        docker_password=docker_password,
        docker_build_repository=docker_build_repository,
        docker_release_repository=docker_release_repository,
        commit_sha=commit_sha,
        build_config=build_config,
        git_access=GitAccess(),
    )



from exasol.slc_ci.model.build_config_model import BuildConfig
from exasol.slc_ci.model.flavor_ci_model import FlavorCiConfig


def get_flavor_ci_model(build_config: BuildConfig, flavor: str) -> FlavorCiConfig:
    flavor_ci_config_path = build_config.flavors_path / flavor / "ci.json"
    return FlavorCiConfig.model_validate_json(
        flavor_ci_config_path.read_text(), strict=True
    )



import os
from pathlib import Path


class GithubAccess:
    def __init__(self, github_output_var: str):
        self.github_output_var = github_output_var

    @property
    def _get_github_output_file(self):
        get_github_output_file = Path(os.getenv("GITHUB_OUTPUT"))
        return get_github_output_file

    def write_result(self, value: str):
        with open(self._get_github_output_file, "a") as f:
            f.write(f"{self.github_output_var}={value}\n")



import logging
from pathlib import Path
from typing import List

from exasol.slc_ci.lib.branch_config import build_always
from exasol.slc_ci.lib.get_build_config_model import get_build_config_model
from exasol.slc_ci.lib.git_access import GitAccess
from exasol.slc_ci.lib.github_access import GithubAccess


def get_all_affected_files(
    git_access: GitAccess, base_ref: str, remote: str
) -> List[Path]:
    complete_base_ref = f"{remote}/{base_ref}" if remote else base_ref
    base_last_commit_sha = git_access.get_head_commit_sha_of_branch(complete_base_ref)
    changed_files = set()  # type: ignore
    for commit in git_access.get_last_commits():
        if commit == base_last_commit_sha:
            break
        changed_files.update(git_access.get_files_of_commit(commit))
    return [Path(changed_file) for changed_file in changed_files]


def _run_check_if_need_to_build(
    branch_name: str, base_ref: str, remote: str, flavor: str, git_access: GitAccess
) -> bool:
    build_config = get_build_config_model()
    if build_always(branch_name):
        return True
    if "[rebuild]" in git_access.get_last_commit_message():
        return True
    affected_files = list(get_all_affected_files(git_access, base_ref, remote))
    logging.debug(
        f"check_if_need_to_build: Found files of last commits: {affected_files}"
    )
    affected_files = [
        file
        for file in affected_files
        if not any(
            [
                file.is_relative_to(ignore_path)
                for ignore_path in build_config.ignore_paths
            ]
        )
    ]

    if len(affected_files) > 0:
        # Now filter out also other flavor folders
        this_flavor_path = build_config.flavors_path / flavor
        affected_files = [
            file
            for file in affected_files
            if (
                file.is_relative_to(this_flavor_path)
                or not file.is_relative_to(build_config.flavors_path)
            )
        ]
    logging.debug(f"check_if_need_to_build: filtered files: {affected_files}")
    return len(affected_files) > 0


def check_if_need_to_build(
    branch_name: str,
    base_ref: str,
    remote: str,
    flavor: str,
    github_access: GithubAccess,
    git_access: GitAccess,
) -> None:
    res = _run_check_if_need_to_build(branch_name, base_ref, remote, flavor, git_access)
    github_access.write_result("True" if res else "False")



import logging
import re
from tempfile import TemporaryDirectory
from typing import Tuple

from exasol_script_languages_container_ci.lib.asset_uploader import AssetUploader
from exasol_script_languages_container_ci.lib.ci_export import CIExport


def _parse_repo_url(source_repo_url: str) -> str:
    """
    source_repo_url is expected to have the following format: `https://github.com/exasol/script-languages-repo`
    where `exasol/script-languages-repo` is the repository for which the release will be created.
    This method returns the repository id.
    """
    res = re.search(r"^https://github.com/([a-zA-Z0-9\-_/]+)$", source_repo_url)
    if res is None:
        raise ValueError(
            f"Parameter source_repo_url={source_repo_url} does not match the following regex: "
            rf"^https://github.com/([a-zA-Z0-9\-_/]+)$"
        )
    return res.groups()[0]


class ReleaseUploader:

    def __init__(self, asset_uploader: AssetUploader, ci_export: CIExport = CIExport()):
        self._ci_export = ci_export
        self._asset_uploader = asset_uploader

    def release_upload(
        self, flavor_path: Tuple[str, ...], source_repo_url: str, release_id: int
    ) -> None:
        """
        Exports the container into tar.gz(s) and uploads to the repository / release.
        release_key is expected to have the following format: "{key}:{value}" where {key} can be:
        * "Tag"
        * "Id"
        source_repo_url is expected to have the following format: `https://github.com/exasol/script-languages-repo`
        """
        repo_id = _parse_repo_url(source_repo_url)
        with TemporaryDirectory() as temp_dir:
            logging.info(f"Running command 'export' with parameters: {locals()}")
            self._ci_export.export(flavor_path=flavor_path, export_path=temp_dir)
            self._asset_uploader.upload_assets(
                repo_id=repo_id,
                release_id=release_id,
                content_type="application/gzip",
                artifact_path=temp_dir,
                file_suffix=".tar.gz",
                label_prefix="Flavor",
            )
            self._asset_uploader.upload_assets(
                repo_id=repo_id,
                release_id=release_id,
                content_type="text/plain",
                artifact_path=temp_dir,
                file_suffix=".tar.gz.sha512sum",
                label_prefix="Checksum",
            )



import click
from exasol_integration_test_docker_environment.lib.utils.cli_function_decorators import (
    add_options,
)

import exasol.slc_ci.lib.check_if_build_needed as lib_check_if_build_needed
from exasol.slc_ci.cli.cli import cli
from exasol.slc_ci.cli.options.branch_options import (
    base_branch_option,
    branch_option,
    remote_option,
)
from exasol.slc_ci.cli.options.flavor_options import flavor_options
from exasol.slc_ci.cli.options.github_options import github_options
from exasol.slc_ci.lib.git_access import GitAccess
from exasol.slc_ci.lib.github_access import GithubAccess


@cli.command()
@add_options(flavor_options)
@add_options([branch_option, base_branch_option, remote_option])
@add_options(github_options)
def check_if_build_needed(
    flavor: str, branch_name: str, base_ref: str, remote: str, github_output_var: str
) -> None:
    git_access: GitAccess = GitAccess()
    github_access: GithubAccess = GithubAccess(github_output_var)
    lib_check_if_build_needed.check_if_need_to_build(
        branch_name=branch_name,
        base_ref=base_ref,
        remote=remote,
        flavor=flavor,
        github_access=github_access,
        git_access=git_access,
    )



import re
from dataclasses import dataclass


@dataclass(frozen=True)
class BuildActions:
    build_always: bool
    rebuild: bool
    push_to_docker_release_repo: bool


@dataclass(frozen=True)
class BranchConfig:
    develop = BuildActions(
        build_always=True, rebuild=True, push_to_docker_release_repo=False
    )
    main = BuildActions(
        build_always=True, rebuild=True, push_to_docker_release_repo=True
    )
    rebuild = BuildActions(
        build_always=True, rebuild=True, push_to_docker_release_repo=False
    )
    other = BuildActions(
        build_always=False, rebuild=False, push_to_docker_release_repo=False
    )


def _get_branch_config(branch_name: str) -> BuildActions:
    matches = (
        (re.compile(r"refs/heads/(master|main)"), BranchConfig.main),
        (re.compile(r"refs/heads/develop"), BranchConfig.develop),
        (re.compile(r"refs/heads/rebuild/.*"), BranchConfig.rebuild),
    )

    branch_cfg = BranchConfig.other
    for branch_regex, branch_config in matches:
        if branch_regex.match(branch_name):
            branch_cfg = branch_config
            break
    return branch_cfg


def build_always(branch_name: str) -> bool:
    return _get_branch_config(branch_name).build_always


def rebuild(branch_name) -> bool:
    return _get_branch_config(branch_name).rebuild


def push_to_docker_release_repo(branch_name: str) -> bool:
    return _get_branch_config(branch_name).push_to_docker_release_repo



import jinja2


def render_template(template: str, **kwargs):
    env = jinja2.Environment(
        loader=jinja2.PackageLoader("exasol_script_languages_container_ci"),
        autoescape=jinja2.select_autoescape(),
        keep_trailing_newline=True,
    )
    t = env.get_template(template)
    return t.render(**kwargs)



from exasol.slc_ci.lib.get_build_config_model import get_build_config_model
from exasol.slc_ci.lib.get_flavor_ci_model import get_flavor_ci_model
from exasol.slc_ci.lib.github_access import GithubAccess


def get_build_runner(flavor: str, github_access: GithubAccess):
    build_config = get_build_config_model()
    flavor_config = get_flavor_ci_model(build_config, flavor)
    github_access.write_result(flavor_config.build_runner)



import json
import logging
from pathlib import Path
from tempfile import TemporaryDirectory

from datamodel_code_generator import InputFileType, generate

from exasol_script_languages_container_ci.lib.render_template import render_template

logger = logging.getLogger(__name__)

CONFIG_DATA_MODEL_FILE_NAME = "config_data_model.py"


def config_data_model_default_output_file() -> Path:
    return Path(__file__).parent / CONFIG_DATA_MODEL_FILE_NAME


def generate_config_data_model(output_file: Path) -> Path:
    schema_dict = json.loads(render_template("config_schema.json"))
    schema_json = json.dumps(schema_dict)
    with TemporaryDirectory() as directory:
        temp_output_file = Path(directory) / CONFIG_DATA_MODEL_FILE_NAME
        generate(
            schema_json,
            input_file_type=InputFileType.JsonSchema,
            output=temp_output_file,
            class_name="Config",
            apply_default_values_for_required_fields=True,
        )
        with temp_output_file.open("rt") as temp_output_file_handle:
            with output_file.open("wt") as output_file_handle:
                lines = (line for line in temp_output_file_handle)
                lines = filter(lambda line: "#   timestamp: " not in line, lines)  # type: ignore
                for line in lines:
                    output_file_handle.write(line)
    return output_file


def regenerate_config_data_model(output_file: Path):
    with TemporaryDirectory() as directory:
        temp_output_file = Path(directory) / CONFIG_DATA_MODEL_FILE_NAME
        generate_config_data_model(temp_output_file)
        with temp_output_file.open("rt") as file:
            new_model = file.read()
        if output_file.is_file():
            with output_file.open("rt") as file:
                old_model = file.read()
            if new_model != old_model:
                logger.warning(f"Regenerating config pydantic model to {output_file}")
                with output_file.open("wt") as file:
                    file.write(new_model)
        else:
            logger.info(f"Generating config pydantic model to new file {output_file}")
            with output_file.open("wt") as file:
                file.write(new_model)



import logging
from typing import Optional, Tuple

from exasol.slc.api import build
from exasol.slc.internal.tasks.test.test_container_content import (
    build_test_container_content,
)
from exasol_integration_test_docker_environment.lib.api.build_test_container import (
    build_test_container,
)

from exasol_script_languages_container_ci.lib.ci_step_output_printer import (
    CIStepOutputPrinter,
    CIStepOutputPrinterProtocol,
)


class CIBuild:

    def __init__(
        self, printer: CIStepOutputPrinterProtocol = CIStepOutputPrinter(logging.info)
    ):
        self._printer = printer

    def build(
        self,
        flavor_path: Tuple[str, ...],
        rebuild: bool,
        build_docker_repository: Optional[str],
        commit_sha: str,
        docker_user: str,
        docker_password: str,
        test_container_folder: str,
    ):
        """
        Build the script-language container for given flavor. And also build the test container
        """

        logging.info(f"Running command 'build' with parameters: {locals()}")
        if build_docker_repository is None:
            slc_image_infos = build(
                flavor_path=flavor_path,
                force_rebuild=rebuild,
                source_docker_tag_prefix=commit_sha,
                source_docker_username=docker_user,
                source_docker_password=docker_password,
                shortcut_build=False,
                workers=7,
                log_level="WARNING",
                use_job_specific_log_file=True,
            )
        else:
            slc_image_infos = build(
                flavor_path=flavor_path,
                force_rebuild=rebuild,
                source_docker_repository_name=build_docker_repository,
                source_docker_tag_prefix=commit_sha,
                source_docker_username=docker_user,
                source_docker_password=docker_password,
                shortcut_build=False,
                workers=7,
                log_level="WARNING",
                use_job_specific_log_file=True,
            )
        logging.info(
            f"Running command 'build_test_container' with parameters: {locals()}"
        )
        content = build_test_container_content(test_container_folder)
        test_container_image_infos = build_test_container(
            force_rebuild=rebuild,
            workers=7,
            test_container_content=content,
            source_docker_repository_name=build_docker_repository,
            source_docker_tag_prefix=commit_sha,
            log_level="WARNING",
            use_job_specific_log_file=True,
        )
        self._printer.print_exasol_docker_images()



from typing import List, Optional

from exasol.slc.models.accelerator import Accelerator
from pydantic import BaseModel


class TestSet(BaseModel):
    name: str
    folders: List[str]
    goal: str
    generic_language_tests: List[str]
    test_runner: Optional[str] = None
    accelerator: Accelerator = Accelerator.NONE


class TestConfig(BaseModel):
    default_test_runner: str
    test_sets: List[TestSet]


class FlavorCiConfig(BaseModel):
    build_runner: str
    test_config: TestConfig



import re
from enum import Enum, auto


class BuildSteps(Enum):
    BUILD_ALL_ALWAYS = auto()
    REBUILD = auto()
    PUSH_TO_DOCKER_RELEASE_REPO = auto()


class BranchConfig(Enum):
    DEVELOP = {
        BuildSteps.BUILD_ALL_ALWAYS: True,
        BuildSteps.REBUILD: True,
        BuildSteps.PUSH_TO_DOCKER_RELEASE_REPO: False,
    }
    MAIN = {
        BuildSteps.BUILD_ALL_ALWAYS: True,
        BuildSteps.REBUILD: True,
        BuildSteps.PUSH_TO_DOCKER_RELEASE_REPO: True,
    }
    REBUILD = {
        BuildSteps.BUILD_ALL_ALWAYS: True,
        BuildSteps.REBUILD: True,
        BuildSteps.PUSH_TO_DOCKER_RELEASE_REPO: False,
    }
    OTHER = {
        BuildSteps.BUILD_ALL_ALWAYS: False,
        BuildSteps.REBUILD: False,
        BuildSteps.PUSH_TO_DOCKER_RELEASE_REPO: False,
    }

    @staticmethod
    def build_always(branch_name: str) -> bool:
        return get_branch_config(branch_name).value[BuildSteps.BUILD_ALL_ALWAYS]

    @staticmethod
    def rebuild(branch_name) -> bool:
        return get_branch_config(branch_name).value[BuildSteps.REBUILD]

    @staticmethod
    def push_to_docker_release_repo(branch_name: str) -> bool:
        return get_branch_config(branch_name).value[
            BuildSteps.PUSH_TO_DOCKER_RELEASE_REPO
        ]


def get_branch_config(branch_name: str) -> BranchConfig:
    matches = (
        (re.compile(r"refs/heads/(master|main)"), BranchConfig.MAIN),
        (re.compile(r"refs/heads/develop"), BranchConfig.DEVELOP),
        (re.compile(r"refs/heads/rebuild/.*"), BranchConfig.REBUILD),
    )

    branch_cfg = BranchConfig.OTHER
    for branch_regex, branch_config in matches:
        if branch_regex.match(branch_name):
            branch_cfg = branch_config
            break
    return branch_cfg



import json
import logging
import shutil
from pathlib import Path
from typing import Tuple

from exasol.slc_ci.lib import branch_config
from exasol.slc_ci.lib.branch_config import BranchConfig
from exasol.slc_ci.lib.ci_build import CIBuild
from exasol.slc_ci.lib.ci_export import CIExport
from exasol.slc_ci.lib.ci_prepare import CIPrepare
from exasol.slc_ci.lib.ci_push import CIPush
from exasol.slc_ci.lib.ci_security_scan import CISecurityScan
from exasol.slc_ci.lib.get_build_config_model import get_build_config_model
from exasol.slc_ci.lib.git_access import GitAccess
from exasol.slc_ci.lib.github_access import GithubAccess
from exasol.slc_ci.model.build_config_model import BuildConfig


def _export_slc(
    ci_export: CIExport, github_access: GithubAccess, flavor_path: Tuple[str, ...]
) -> None:
    release_output = ".build_output_release"
    slc_release = ci_export.export(
        flavor_path=flavor_path, goal="release", output_directory=release_output
    )
    test_output = ".build_output_test"
    slc_test = ci_export.export(
        flavor_path=flavor_path,
        goal="base_test_build_run",
        output_directory=test_output,
    )
    github_access.write_result(
        json.dumps(
            {
                "slc_release": {"path": str(slc_release), "goal": "release"},
                "slc_test": {"path": str(slc_test), "goal": "base_test_build_run"},
            }
        )
    )


def _export_and_scan_vulnerabilities_ci(
    flavor: str,
    branch_name: str,
    docker_user: str,
    docker_password: str,
    commit_sha: str,
    git_access: GitAccess,
    github_access: GithubAccess,
    ci_build: CIBuild = CIBuild(),
    ci_security_scan: CISecurityScan = CISecurityScan(),
    ci_prepare: CIPrepare = CIPrepare(),
    ci_export: CIExport = CIExport(),
    ci_push: CIPush = CIPush(),
) -> None:
    logging.info(
        f"Running build image and scan vulnerabilities for parameters: {locals()}"
    )
    build_config: BuildConfig = get_build_config_model()

    flavor_path = (f"{build_config.flavors_path}/{flavor}",)
    test_container_folder = build_config.test_container_folder
    rebuild = branch_config.rebuild(branch_name)
    ci_prepare.prepare(commit_sha=commit_sha)
    ci_build.build(
        flavor_path=flavor_path,
        rebuild=rebuild,
        build_docker_repository=build_config.docker_build_repository,
        docker_user=docker_user,
        docker_password=docker_password,
    )
    ci_security_scan.run_security_scan(flavor_path=flavor_path)
    ci_push.push(
        flavor_path=flavor_path,
        target_docker_repository=build_config.docker_build_repository,
        target_docker_tag_prefix=commit_sha,
        docker_user=docker_user,
        docker_password=docker_password,
    )
    ci_push.push(
        flavor_path=flavor_path,
        target_docker_repository=build_config.docker_build_repository,
        target_docker_tag_prefix="",
        docker_user=docker_user,
        docker_password=docker_password,
    )
    _export_slc(ci_export, github_access, flavor_path)


def _export_and_scan_vulnerabilities_cd(
    flavor: str,
    branch_name: str,
    docker_user: str,
    docker_password: str,
    commit_sha: str,
    git_access: GitAccess,
    github_access: GithubAccess,
    ci_build: CIBuild = CIBuild(),
    ci_security_scan: CISecurityScan = CISecurityScan(),
    ci_prepare: CIPrepare = CIPrepare(),
    ci_export: CIExport = CIExport(),
    ci_push: CIPush = CIPush(),
) -> None:
    logging.info(
        f"Running build image and scanning vulnerabilities for release for parameters: {locals()}"
    )
    build_config: BuildConfig = get_build_config_model()

    flavor_path = (f"{build_config.flavors_path}/{flavor}",)
    test_container_folder = build_config.test_container_folder
    ci_prepare.prepare(commit_sha=commit_sha)
    ci_build.build(
        flavor_path=flavor_path,
        rebuild=True,
        build_docker_repository=build_config.docker_build_repository,
        docker_user=docker_user,
        docker_password=docker_password,
    )
    ci_security_scan.run_security_scan(flavor_path=flavor_path)
    ci_push.push(
        flavor_path=flavor_path,
        target_docker_repository=build_config.docker_release_repository,
        target_docker_tag_prefix="",
        docker_user=docker_user,
        docker_password=docker_password,
    )
    _export_slc(ci_export, github_access, flavor_path)


def export_and_scan_vulnerabilities(release: bool = False, **kwargs) -> None:
    if release:
        _export_and_scan_vulnerabilities_cd(**kwargs)
    else:
        _export_and_scan_vulnerabilities_ci(**kwargs)



import glob
import logging
from pathlib import Path

from exasol_script_languages_container_ci.lib.github_release_asset_uploader import (
    GithubReleaseAssetUploader,
)


class AssetUploader:

    def __init__(self, release_asset_uploader: GithubReleaseAssetUploader):
        self._release_asset_uploader = release_asset_uploader

    def upload_assets(
        self,
        repo_id: str,
        release_id: int,
        content_type: str,
        artifact_path: str,
        file_suffix: str,
        label_prefix: str,
    ):
        release_artifacts = glob.glob(f"{artifact_path}/*{file_suffix}")
        for release_artifact in release_artifacts:
            artifact_file_name = Path(release_artifact).name
            if artifact_file_name.endswith(file_suffix):
                artifact_file_name = artifact_file_name[: -len(file_suffix)]
            else:
                logging.error(
                    f"Artifact file: {artifact_file_name} does not end with {file_suffix}. "
                    f"Using {artifact_file_name} as label."
                )
            self._release_asset_uploader.upload(
                archive_path=release_artifact,
                label=f"{label_prefix} {artifact_file_name}",
                repo_id=repo_id,
                release_id=release_id,
                content_type=content_type,
            )



import logging
import os

import click

from exasol_script_languages_container_ci.cli.cli import cli
from exasol_script_languages_container_ci.lib.asset_uploader import AssetUploader
from exasol_script_languages_container_ci.lib.config.config_data_model import Config
from exasol_script_languages_container_ci.lib.github_release_asset_uploader import (
    GithubReleaseAssetUploader,
)
from exasol_script_languages_container_ci.lib.release import release
from exasol_script_languages_container_ci.lib.release_uploader import ReleaseUploader


@cli.command()
@click.option("--flavor", required=True, type=str, help="Flavor name.")
@click.option("--docker-user", required=True, type=str, help="Docker user name")
@click.option("--docker-password", required=True, type=str, help="Docker password")
@click.option(
    "--docker-release-repository",
    required=True,
    type=str,
    help="Docker release repository",
)
@click.option(
    "--config-file",
    required=True,
    type=click.Path(exists=True, file_okay=True, dir_okay=False),
    help="The build config file (project specific)",
)
@click.option(
    "--source-repo-url",
    required=True,
    type=str,
    help="The url of the repository. Usually set by AWS under env variable CODEBUILD_SOURCE_REPO_URL.",
)
@click.option("--release-id", required=True, type=int, help="The id of the release.")
@click.option(
    "--dry-run/--no-dry-run",
    default=False,
    help="If true, runs release without pushing the container to the docker release repository."
    "If false, also pushes the container to the docker release repository.",
)
@click.pass_context
def run_release(
    ctx: click.Context,
    flavor: str,
    docker_user: str,
    docker_password: str,
    docker_release_repository: str,
    config_file: str,
    source_repo_url: str,
    release_id: int,
    dry_run: bool,
):
    logging.basicConfig(level=logging.INFO)
    github_release_asset_uploader = GithubReleaseAssetUploader(
        os.getenv("GITHUB_TOKEN")
    )
    asset_uploader = AssetUploader(release_asset_uploader=github_release_asset_uploader)
    release_uploader = ReleaseUploader(asset_uploader=asset_uploader)
    build_config = Config.parse_file(config_file)
    release(
        flavor=flavor,
        docker_user=docker_user,
        docker_password=docker_password,
        docker_release_repository=docker_release_repository,
        build_config=build_config,
        source_repo_url=source_repo_url,
        release_id=release_id,
        release_uploader=release_uploader,
        is_dry_run=dry_run,
    )



import click

docker_options = [
    click.option(
        "--docker-user",
        type=str,
        required=True,
        help="Docker user name",
    ),
    click.option(
        "--docker-password",
        type=str,
        required=True,
        help="Docker password.",
    ),
]



import pathlib
from pathlib import Path
from typing import List

from pydantic import BaseModel


class BuildConfig(BaseModel):

    @property
    def flavors_path(self):
        return Path("flavors")

    ignore_paths: List[pathlib.Path]

    docker_build_repository: str
    docker_release_repository: str
    test_container_folder: str



from inspect import cleandoc
from pathlib import Path
from typing import Callable, Protocol

import docker


class CIStepOutputPrinterProtocol(Protocol):

    def print_exasol_docker_images(self):
        raise NotImplementedError()

    def print_file(self, filename: Path):
        raise NotImplementedError()


def _get_exasol_docker_images():
    docker_client = docker.from_env()
    try:
        exa_images = [
            str(img) for img in docker_client.images.list() if "exasol" in str(img)
        ]
        return exa_images
    finally:
        docker_client.close()


class CIStepOutputPrinter(CIStepOutputPrinterProtocol):

    def __init__(self, writer: Callable[[str], None]):
        self._writer = writer

    def print_exasol_docker_images(self):
        """
        Prints all docker images with "exasol" in it's name to stdout.
        :return: None
        """

        self._writer(
            cleandoc(
                """
            {seperator}
            Printing docker images
            {seperator}
            {images}"""
            ).format(seperator=20 * "=", images="\n".join(_get_exasol_docker_images()))
        )

    def print_file(self, filename: Path):
        """
        Print the file's content to the writer.
        """
        with open(filename) as f:
            self._writer(f.read())



import json

from exasol.slc_ci.lib.get_build_config_model import get_build_config_model
from exasol.slc_ci.lib.get_flavor_ci_model import get_flavor_ci_model
from exasol.slc_ci.lib.github_access import GithubAccess
from exasol.slc_ci.model.flavor_ci_model import FlavorCiConfig, TestSet


def _build_test_matrix_entry(flavor_config: FlavorCiConfig, test_set: TestSet) -> dict:
    return {
        "test-set-name": test_set.name,
        "test-runner": (
            test_set.test_runner
            if test_set.test_runner
            else flavor_config.test_config.default_test_runner
        ),
        "goal": test_set.goal,
    }


def get_test_matrix(flavor: str, github_access: GithubAccess):
    build_config = get_build_config_model()
    flavor_config = get_flavor_ci_model(build_config, flavor)
    test_matrix = {
        "include": [
            _build_test_matrix_entry(flavor_config, entry)
            for entry in flavor_config.test_config.test_sets
        ]
    }
    github_access.write_result(json.dumps(test_matrix))



import logging

from exasol_script_languages_container_ci.lib.ci_build import CIBuild
from exasol_script_languages_container_ci.lib.ci_prepare import CIPrepare
from exasol_script_languages_container_ci.lib.ci_push import CIPush
from exasol_script_languages_container_ci.lib.ci_security_scan import CISecurityScan
from exasol_script_languages_container_ci.lib.ci_test import CIExecuteTest
from exasol_script_languages_container_ci.lib.config.config_data_model import Config
from exasol_script_languages_container_ci.lib.release_uploader import ReleaseUploader


def release(
    flavor: str,
    docker_user: str,
    docker_password: str,
    docker_release_repository: str,
    build_config: Config,
    source_repo_url: str,
    release_id: int,
    is_dry_run: bool,
    release_uploader: ReleaseUploader,
    ci_build: CIBuild = CIBuild(),
    ci_execute_tests: CIExecuteTest = CIExecuteTest(),
    ci_push: CIPush = CIPush(),
    ci_security_scan: CISecurityScan = CISecurityScan(),
    ci_prepare: CIPrepare = CIPrepare(),
):
    """
    Run Release build:
    1. Build image
    2. Run basic tests
    3. Push to docker release repository
    4. Upload to GH release url
    """
    logging.info(f"Running Release build for parameters: {locals()}")

    flavor_path = (f"flavors/{flavor}",)
    test_container_folder = "test_container"
    ci_prepare.prepare()
    ci_build.build(
        flavor_path=flavor_path,
        rebuild=True,
        build_docker_repository=None,
        commit_sha="",
        docker_user=None,  # type: ignore
        docker_password=None,  # type: ignore
        test_container_folder=test_container_folder,
    )
    ci_execute_tests.execute_tests(
        flavor_path=flavor_path,
        docker_user=docker_user,
        docker_password=docker_password,
        test_container_folder=test_container_folder,
    )
    ci_security_scan.run_security_scan(flavor_path=flavor_path)
    if not is_dry_run:
        ci_push.push(
            flavor_path=flavor_path,
            target_docker_repository=docker_release_repository,
            target_docker_tag_prefix="",
            docker_user=docker_user,
            docker_password=docker_password,
        )
    else:
        logging.info("Skipping push to docker release repository due to dry-run.")
    release_uploader.release_upload(
        flavor_path=flavor_path, source_repo_url=source_repo_url, release_id=release_id
    )
